module Plant
  # Main Plant Methods
  module Core
    extend ActiveSupport::Concern
    included do
      # Can only have one scenario-node_id combination
      validates_uniqueness_of :node_id
    end

    # All methods are class level, extend this class to allow for instance
    # level variables
    module ClassMethods
      # Gets a given node
      #   => Parameter: node ID (string)
      #   => Parameter: Options (hash), containing
      #      - scenario (optional)
      #        Can either be a string or an object with
      #        a scenario method, for example a user.
      #
      #        If no specific scenario returned, then will
      #        default back to a nil scenario.
      #
      #   => Throws: NotFound if node not found
      #   => Returns: A content object
      def get_node(node_id, options = {})
        options = defaults.merge(options)
        find_specific(node_id, options[:scenario]) || find_main(node_id)
      end

      def defaults
        { scenario: nil,
          parameters: {} }
      end

      # Convenience method to get the content of a given node
      #  => Paramter: node ID (string)
      #  => Paramter: Options (hash)
      #   - scenario
      #     Can either be a string or an object with
      #     a scenario method, for example a user
      #
      #   - Also can be any arbitary key to be injected into the
      #     returned content. For example
      #     When called get_node_content(node_id, foo: 'hhh)
      #     "This is content for #{foo} and bar"
      #     will return 'This is content for hhh and bar'
      #
      #  Note: If second parameter is a string, it will be treated as
      #  scenario.
      # Also see: get_node
      def get_node_content(node_id, options = {})
        options = { scenario: options } unless options.is_a? Hash
        options = defaults.merge(options)
        inject_content(get_node(node_id, options).content, options)
      end

      # Interpolation of content
      def inject_content(content, options)
        options.each do |key, replacement|
          content.gsub!("\#\{#{key}\}", replacement.to_s)
        end
        content
      end

      # Private
      # Finds a specific scenario
      def find_specific(node, scenario)
        scenario = scenario.scenario if scenario.respond_to? :scenario
        find_by(node_id: "#{node}.#{scenario}")
      end

      # Private
      # Find the main node or falls back to the node if main does not exist.
      def find_main(node)
        main = find_by(node_id: "#{node}.main")
        return main if main
        find_by!(node_id: "#{node}")
      end
    end
  end
end
